// -*- C++ -*-
/*!
 * @file  TileGenerator.cpp
 * @brief 画像付加データDBから画像付加データを取得しタイル画像を作成する
 * @date $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * New BSD
 *
 * $Id$
 */

#include "TileGenerator.h"
#include <opencv2\opencv.hpp>
#include <iostream>
#include <fstream>
#include <direct.h>
#include <time.h>

#define _USE_MATH_DEFINES
#include <math.h>  


// Module specification
// <rtc-template block="module_spec">
static const char* tilegenerator_spec[] =
  {
    "implementation_id", "TileGenerator",
    "type_name",         "TileGenerator",
    "description",       "画像付加データDBから画像付加データを取得しタイル画像を作成する",
    "version",           "0.5",
    "vendor",            "EAC",
    "category",          "Image Control",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_Image_Resize", "1",
    "conf.default.02_Offset_Width", "0",
    "conf.default.03_Offset_Height", "0",
    "conf.default.04_TileX_Coord_Start", "0",
    "conf.default.05_TileY_Coord_Start", "0",
    "conf.default.06_ZLvl18_Alt_Min", "30",
    "conf.default.07_ZLvl18_Alt_Max", "49",
    "conf.default.08_ZLvl19_Alt_Min", "50",
    "conf.default.09_ZLvl19_Alt_Max", "69",
    "conf.default.10_ZLvl20_Alt_Min", "70",
    "conf.default.11_ZLvl20_Alt_Max", "89",
    "conf.default.12_ZLvl21_Alt_Min", "90",
    "conf.default.13_ZLvl21_Alt_Max", "110",
    "conf.default.14_View_Angle", "31.5",
    // Widget
    "conf.__widget__.01_Image_Resize", "spin.0.1",
    "conf.__widget__.02_Offset_Width", "spin.1",
    "conf.__widget__.03_Offset_Height", "spin.1",
    "conf.__widget__.04_TileX_Coord_Start", "spin.1",
    "conf.__widget__.05_TileY_Coord_Start", "spin.1",
    "conf.__widget__.06_ZLvl18_Alt_Min", "spin.1",
    "conf.__widget__.07_ZLvl18_Alt_Max", "spin.1",
    "conf.__widget__.08_ZLvl19_Alt_Min", "spin.1",
    "conf.__widget__.09_ZLvl19_Alt_Max", "spin.1",
    "conf.__widget__.10_ZLvl20_Alt_Min", "spin.1",
    "conf.__widget__.11_ZLvl20_Alt_Max", "spin.1",
    "conf.__widget__.12_ZLvl21_Alt_Min", "spin.1",
    "conf.__widget__.13_ZLvl21_Alt_Max", "spin.1",
    "conf.__widget__.14_View_Angle", "spin.0.1",
    // Constraints
    "conf.__constraints__.01_Image_Resize", "0<x<=2.0",
    "conf.__constraints__.02_Offset_Width", "0<=x<=256",
    "conf.__constraints__.03_Offset_Height", "0<=x<=256",
    "conf.__constraints__.04_TileX_Coord_Start", "0<=x<=4294967294",
    "conf.__constraints__.05_TileY_Coord_Start", "0<=x<=4294967294",
    "conf.__constraints__.06_ZLvl18_Alt_Min", "0<=x<=150",
    "conf.__constraints__.07_ZLvl18_Alt_Max", "0<=x<=150",
    "conf.__constraints__.08_ZLvl19_Alt_Min", "0<=x<=150",
    "conf.__constraints__.09_ZLvl19_Alt_Max", "0<=x<=150",
    "conf.__constraints__.10_ZLvl20_Alt_Min", "0<=x<=150",
    "conf.__constraints__.11_ZLvl20_Alt_Max", "0<=x<=150",
    "conf.__constraints__.12_ZLvl21_Alt_Min", "0<=x<=150",
    "conf.__constraints__.13_ZLvl21_Alt_Max", "0<=x<=150",
    "conf.__constraints__.14_View_Angle", "0<x<=180.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
TileGenerator::TileGenerator(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inCoordAdditinalDataIn("inCoordAdditinalData", m_inCoordAdditinalData),
    m_TileGenerateReqServPort("TileGenerateReqServ"),
    m_TileGenerateStsServPort("TileGenerateStsServ"),
    m_AdditionalDBReqServPort("AdditionalDBReqServ"),
    m_AdditionalDBStsServPort("AdditionalDBStsServ"),
    m_MapDataDBReqServPort("MapDataDBReqServ"),
    m_MapDataDBStsServPort("MapDataDBStsServ")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
TileGenerator::~TileGenerator()
{
}


/*!
 * タイル生成RTCの初期化処理
 */
RTC::ReturnCode_t TileGenerator::onInitialize()
{
#ifdef _DEBUG
	std::cout << "[debug]TileGenerator::onInitialize" << std::endl;
#endif
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inCoordAdditinalData", m_inCoordAdditinalDataIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_TileGenerateReqServPort.registerProvider("TileGenerateReqMakeTile", "TileGenerateReqServ::TileGenerateReqMakeTile", m_TileGenerateReqMakeTileProv);
  m_TileGenerateStsServPort.registerProvider("TileGenerateNotice", "TileGenerateStsServ::TileGenerateNotice", m_TileGenerateNoticeProv);
  
  // Set service consumers to Ports
  m_AdditionalDBReqServPort.registerConsumer("CoordinateDataGet", "AdditionalDBReqServ::CoordinateDataGet", m_CoordinateDataGetCons);
  m_AdditionalDBStsServPort.registerConsumer("AddDBNotice", "AdditionalDBStsServ::AddDBNotice", m_AddDBNoticeCons);
  m_MapDataDBReqServPort.registerConsumer("MapDBReqRegistAddData", "MapDataDBReqServ::MapDBReqRegistAddData", m_MapDBReqRegistAddDataCons);
  m_MapDataDBStsServPort.registerConsumer("MapDataDBNotice", "MapDataDBStsServ::MapDataDBNotice", m_MapDataDBNoticeCons);
  
  // Set CORBA Service Ports
  addPort(m_TileGenerateReqServPort);
  addPort(m_TileGenerateStsServPort);
  addPort(m_AdditionalDBReqServPort);
  addPort(m_AdditionalDBStsServPort);
  addPort(m_MapDataDBReqServPort);
  addPort(m_MapDataDBStsServPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("01_Image_Resize", m_fImage_Resize, "1");
  bindParameter("02_Offset_Width", m_lOffset_Width, "0");
  bindParameter("03_Offset_Height", m_lOffset_Height, "0");
  bindParameter("04_TileX_Coord_Start", m_lTileX_Coord_Start, "0");
  bindParameter("05_TileY_Coord_Start", m_lTileY_Coord_Start, "0");
  bindParameter("06_ZLvl18_Alt_Min", m_lZLvl18_Alt_Min, "30");
  bindParameter("07_ZLvl18_Alt_Max", m_lZLvl18_Alt_Max, "49");
  bindParameter("08_ZLvl19_Alt_Min", m_lZLvl19_Alt_Min, "50");
  bindParameter("09_ZLvl19_Alt_Max", m_lZLvl19_Alt_Max, "69");
  bindParameter("10_ZLvl20_Alt_Min", m_lZLvl20_Alt_Min, "70");
  bindParameter("11_ZLvl20_Alt_Max", m_lZLvl20_Alt_Max, "89");
  bindParameter("12_ZLvl21_Alt_Min", m_lZLvl21_Alt_Min, "90");
  bindParameter("13_ZLvl21_Alt_Max", m_lZLvl21_Alt_Max, "110");
  bindParameter("14_View_Angle", m_dView_Angle, "31.5");
  // </rtc-template>
  
  TileGeneratorRamInit();
  m_inCoordAdditinalData.data.length(PLAN_LIST_NUM_MAX);

  return RTC::RTC_OK;
}

/*!
 * タイル生成RTCの終了処理
 */

RTC::ReturnCode_t TileGenerator::onFinalize()
{
#ifdef _DEBUG
	std::cout << "[debug]TileGenerator::onFinalize" << std::endl;
#endif
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t TileGenerator::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TileGenerator::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * タイル生成RTCの開始処理
 */

RTC::ReturnCode_t TileGenerator::onActivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]TileGenerator::onActivated" << std::endl;
#endif
  return RTC::RTC_OK;
}

/*!
 * タイル生成RTCの停止処理
 */

RTC::ReturnCode_t TileGenerator::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]TileGenerator::onDeactivated" << std::endl;
#endif
	TileGeneratorRamInit();
  return RTC::RTC_OK;
}

/*!
 * タイル生成RTCの周期処理
 */

RTC::ReturnCode_t TileGenerator::onExecute(RTC::UniqueId ec_id)
{
	uhword_type uhwResult = RESULT_ERROR;
	bool_type	blRet = FALSE;

	blRet = blTileGeneMatrixTrigerSet();
	if(blRet){
		uhwResult = uhwTileGeneMatrixCheck();
		if(uhwResult == RESULT_SUCCESS){
			uhwResult = uhwTileGeneProcExe();
		}
		if(uhwResult == RESULT_ERROR){
			return RTC::RTC_ERROR;
		}
	}

  return RTC::RTC_OK;
}

/*!
 * タイル生成RTCのエラー遷移処理
 */

RTC::ReturnCode_t TileGenerator::onAborting(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]TileGenerator::onAborting" << std::endl;
#endif
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t TileGenerator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TileGenerator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TileGenerator::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TileGenerator::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*=============================================================================
	テーブル定義
==============================================================================*/

//マトリクステーブル
struct MatrixContens{
	uhword_type	uhwAct;			//アクション
	uhword_type	uhwMode;		//モード
};

struct MatrixContens TileGeneMatrixTbl[][MODE_MAX] ={
	//アイドル											//画像付加データ取得中						//タイル分割確認画面表示中						//地図データDB登録中
	{{ACT_NON,					MODE_NO_CHG			},{ACT_TILE_SPLIT,			MODE_NO_CHG		},{ACT_NON,					MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		}},	//座標指定画像付加データ取得正常終了
	{{ACT_NON,					MODE_NO_CHG			},{ACT_TILE_GENERATE_ERROR,	MODE_IDLE		},{ACT_NON,					MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		}},	//座標指定画像付加データ取得異常終了
	{{ACT_NON,					MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		},{ACT_NON,					MODE_NO_CHG			},{ACT_TILE_GENERATE_COMP,	MODE_IDLE		}},	//地図データDB登録正常終了
	{{ACT_NON,					MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		},{ACT_NON,					MODE_NO_CHG			},{ACT_TILE_GENERATE_ERROR,	MODE_IDLE		}},	//地図データDB登録異常終了
	{{ACT_TILE_GENERATE_START,	MODE_ADD_DATA_GET	},{ACT_NON,					MODE_NO_CHG		},{ACT_TILE_GENERATE_START,	MODE_ADD_DATA_GET	},{ACT_NON,					MODE_NO_CHG		}},	//タイル生成要求
	{{ACT_TILE_GENERATE_START,	MODE_ADD_DATA_GET	},{ACT_NON,					MODE_NO_CHG		},{ACT_TILE_GENERATE_COMP,	MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		}},	//タイル分割確認画面表示要求
	{{ACT_SPLIT_DISP_OFF,		MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		},{ACT_SPLIT_DISP_OFF,		MODE_IDLE			},{ACT_NON,					MODE_NO_CHG		}},	//タイル分割確認画面消去要求
	{{ACT_NON,					MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		},{ACT_SPLIT_DISP_RENEW,	MODE_NO_CHG			},{ACT_NON,					MODE_NO_CHG		}},	//タイル分割確認画面更新要求
};

//マトリクステーブル
struct ConvZLvlToAlt{
	uword_type	uwInZoomLevel;	//入力ズームレベル
	uword_type	uwOutAltMin;	//出力高度Min
	uword_type	uwOutAltMax;	//出力高度Max
};

/*****************************************************************************************
	サブルーチン
******************************************************************************************/
/*****************************************************************************************
*	Function Name		：blTileGeneMatrixTrigerSet
*	Description			：サービスポートの要求をチェックしマトリクスの内部トリガを設定する
*	Argument			：なし
*	Return				：TRUE：トリガあり、FALSE：トリガなし
******************************************************************************************/
bool_type TileGenerator::blTileGeneMatrixTrigerSet(void)
{
	bool_type	blRet = FALSE;
	bool_type	blRet2 = FALSE;
	uhword_type uhwTriger = TRG_NON;

	blRet = blAddDBNoticeTrigCheck(uhwTriger);
	if(blRet == FALSE){
		blRet = blMapDBNoticeTrigCheck(uhwTriger);
		//付加データ取得中、地図データ登録中はトリガ要求のチェックを保留する
		if(((m_uhwMatrixMode == MODE_IDLE) || (m_uhwMatrixMode == MODE_SPLIT_DISP)) && (blRet == FALSE)){
			blRet = blTileGeneTrigCheck(uhwTriger);
		}
	}

	if((blRet == TRUE) && (uhwTriger != TRG_NON)){
		m_uhwMatrixTriger = uhwTriger;
		blRet2 = TRUE;
	}

	return blRet2;
}

/*******************************************************************************
*	Function Name		：uhwTileGeneMatrixCheck
*	Description			：マトリクスチェック処理
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
*******************************************************************************/
uhword_type TileGenerator::uhwTileGeneMatrixCheck(void)
{
	uhword_type uhwMode = MODE_NO_CHG;

	if((m_uhwMatrixTriger >= TRG_MAX) || (m_uhwMatrixMode >= MODE_MAX)){
		return RESULT_ERROR;
	}

	m_uhwMatrixActID = TileGeneMatrixTbl[m_uhwMatrixTriger][m_uhwMatrixMode].uhwAct;
	uhwMode = TileGeneMatrixTbl[m_uhwMatrixTriger][m_uhwMatrixMode].uhwMode;

	//内部モードを更新
	if(uhwMode != MODE_NO_CHG){
		if(m_uhwMatrixMode != uhwMode){
			TileGeneModeSet(uhwMode);
		}
	}

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwTileGeneProcExe
*	Description			：プロセス実行処理
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwTileGeneProcExe(void)
{
	uhword_type uhwRet = RESULT_ERROR;

	//実行処理テーブル
	uhword_type(TileGenerator::*RunProcTable[])(void) ={
		&TileGenerator::uhwProcTileGenerateStart,		//ACT_TILE_GENERATE_START
		&TileGenerator::uhwProcTileSplit,				//ACT_TILE_SPLIT
		&TileGenerator::uhwProcTileGenerateComp,		//ACT_TILE_GENERATE_COMP
		&TileGenerator::uhwProcTileGenerateError,		//ACT_TILE_GENERATE_ERROR
		&TileGenerator::uhwProcSplitDispRenew,			//ACT_SPLIT_DISP_RENEW
		&TileGenerator::uhwProcSplitDispOff,			//ACT_SPLIT_DISP_OFF
	};

	if(m_uhwMatrixActID == ACT_NON){
		//動作要求なしの場合、何もせず正常終了を返す
		return RESULT_SUCCESS;
	}

	//処理実行
	if(m_uhwMatrixActID < ACT_MAX){
		uhwRet = (this->*RunProcTable[m_uhwMatrixActID])();
#ifdef _DEBUG
		std::cout << "[debug]uhwTileGeneProcExe():procid = " << m_uhwMatrixActID <<",Result = " << uhwRet << std::endl;
#endif
	}

	return uhwRet;
}

/*****************************************************************************************
*	Function Name		：blAddDBNoticeTrigCheck
*	Description			：画像付加データDB完了通知をチェックし内部イベントに変換する
*	Argument			：out：内部イベント
*	Return				：TRUE:トリガあり、FALSE：トリガなし
******************************************************************************************/
bool_type TileGenerator::blAddDBNoticeTrigCheck(uhword_type &uhwTriger)
{
	bool_type	blRet = FALSE;
	uword_type	uwNotice = NOTICE_INIT;
	uword_type	uwParam = NOTICE_PARAM_INIT;

	uhwTriger = TRG_NON;

	//サービスポート接続確認
	if(blAddDBNoticeConsChk()){
		m_AddDBNoticeCons->GetAddDBNotice(uwNotice, uwParam);
		if((uwNotice == NOTICE_INIT) && (uwParam == NOTICE_PARAM_INIT)){
			//変化なし
			return blRet;
		}

		switch(uwNotice){
			case ADD_DB_NOTICE_COORD_GET_SUCCESS:
				uhwTriger = TRG_ADDDB_GET_ADD_DATA_SUCCESS;
				break;
			case ADD_DB_NOTICE_COORD_GET_ERROR:
				uhwTriger = TRG_ADDDB_GET_ADD_DATA_ERROR;
				break;
			default:
				break;
		}

		if(uhwTriger != TRG_NON){
			std::cout << "[debug]GetAddDBNotice()" << ",Param1 = " << uwNotice << ",Param2 = " << uwParam << std::endl;
			//完了通知初期化
			m_AddDBNoticeCons->usSetAddDBNotice(NOTICE_INIT, NOTICE_PARAM_INIT);
			m_uwAddDBNotice = uwNotice;
			m_uwAddDBParam = uwParam;
			blRet = TRUE;
		}
	}

	return blRet;
}

/*****************************************************************************************
*	Function Name		：blMapDBNoticeTrigCheck
*	Description			：地図データDB完了通知をチェックし内部イベントに変換する
*	Argument			：out：内部イベント
*	Return				：TRUE:トリガあり、FALSE：トリガなし
******************************************************************************************/
bool_type TileGenerator::blMapDBNoticeTrigCheck(uhword_type &uhwTriger)
{
	bool_type	blRet = FALSE;
	uword_type	uwNotice = NOTICE_INIT;
	uword_type	uwParam = NOTICE_PARAM_INIT;

	uhwTriger = TRG_NON;

	//サービスポート接続確認
	if(blMapDBNoticeConsChk()){
		m_MapDataDBNoticeCons->GetMapDataDBNotice(uwNotice, uwParam);
		if((uwNotice == NOTICE_INIT) && (uwParam == NOTICE_PARAM_INIT)){
			//変化なし
			return blRet;
		}

		switch(uwNotice){
			case MAPDB_REGIST_SUCCESS:
				uhwTriger = TRG_MAPDB_REGIST_SUCCESS;
				break;
			case MAPDB_REGIST_ERROR:
				uhwTriger = TRG_MAPDB_REGIST_ERROR;
				break;
			default:
				break;
		}

		if(uhwTriger != TRG_NON){
			std::cout << "[debug]GetMapDataDBNotice()" << ",Param1 = " << uwNotice << ",Param2 = " << uwParam << std::endl;
			//完了通知初期化
			m_MapDataDBNoticeCons->usSetMapDataDBNotice(NOTICE_INIT, NOTICE_PARAM_INIT);
			m_uwMapDBNotice = uwNotice;
			blRet = TRUE;
		}
	}

	return blRet;
}

/*****************************************************************************************
*	Function Name		：blTileGeneTrigCheck
*	Description			：タイル生成要求をチェックし内部イベントに変換する
*	Argument			：out：内部イベント
*	Return				：TRUE:トリガあり、FALSE：トリガなし
******************************************************************************************/
bool_type TileGenerator::blTileGeneTrigCheck(uhword_type &uhwTriger)
{
	bool_type	blRet = FALSE;
	uword_type uwGenerateMode = TILE_MODE_INIT;
	uword_type uwRobotCategory = ROBOT_CATEGORY_INIT;
	uword_type uwGroupId = GROUP_ID_NON;
	double_type dStartLatitude = 0;
	double_type dStartLongitude = 0;
	double_type dEndLatitude = 0;
	double_type dEndLongitude = 0;
	uword_type uwZoomlevel = ZOOM_LEVEL_DEF;

	uhwTriger = TRG_NON;
	
	uhword_type uhwServResult = RESULT_NACK;

	m_TileGenerateReqMakeTileProv.GetTileGenerateReqMakeTile(uwGenerateMode, uwRobotCategory, uwGroupId, dStartLatitude, dStartLongitude, dEndLatitude, dEndLongitude, uwZoomlevel);
	if((uwGenerateMode != TILE_MODE_INIT) && (uwRobotCategory != ROBOT_CATEGORY_INIT) && (uwGroupId != GROUP_ID_NON) &&
		(dStartLatitude != 0) && (dStartLongitude != 0) && (dEndLatitude != 0) && (dEndLongitude != 0)){
		//各パラメータが初期値以外の場合、有効値と判断し引き続き変化チェックを行う

		if((uwGenerateMode != m_uwGenerateMode) || (uwRobotCategory != m_uwRobotCategory) || (uwGroupId != m_uwGroupId) || (dStartLatitude != m_dStartLatitude) ||
			(dStartLongitude != m_dStartLongitude) || (dEndLatitude != m_dEndLatitude) || (dEndLongitude != m_dEndLongitude) || (uwZoomlevel != m_uwZoomlevel)){
			m_uwGenerateMode = uwGenerateMode;
			m_uwRobotCategory = uwRobotCategory;
			m_uwGroupId = uwGroupId;
			m_dStartLatitude = dStartLatitude;
			m_dStartLongitude = dStartLongitude;
			m_dEndLatitude = dEndLatitude;
			m_dEndLongitude = dEndLongitude;
			m_uwZoomlevel = uwZoomlevel;

			switch(m_uwGenerateMode){
				case TILE_MODE_GENERATE:
					uhwTriger = TRG_REQ_TILE_GENERATE;
					break;
				case TILE_MODE_SPLIT_DISP_ON:
					uhwTriger = TRG_REQ_SPLIT_DISP_ON;
					break;
				case TILE_MODE_SPLIT_DISP_OFF:
					uhwTriger = TRG_REQ_SPLIT_DISP_OFF;
					break;
			}

			if(uhwTriger != TRG_NON){
				std::cout << "[debug]GetTileGenerateReqMakeTile()" <<
					",Param1 = " << uwGenerateMode << ",Param2 = " << uwRobotCategory << ",Param3 = " << uwGroupId << ",Param4 = " << dStartLatitude <<
					",Param5 = " << dStartLongitude << ",Param6 = " << dEndLatitude << ",Param7 = " << dEndLongitude << ",Param8 = " << uwZoomlevel <<std::endl;
				blRet = TRUE;
			}
		}
	}

	//要求なしだが、マトリクスモード状態が分割画面表示中の場合、画像更新要求を行う
	if((blRet == FALSE) && (m_uhwMatrixMode == MODE_SPLIT_DISP)){
		uhwTriger = TRG_REQ_SPLIT_DISP_RENEW;
		blRet = TRUE;
	}

	return blRet;
}

/*****************************************************************************************
*	Function Name		：uhwProcTileGenerateStart
*	Description			：タイル生成を開始する
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwProcTileGenerateStart(void)
{
	uhword_type uhwServResult = RESULT_NACK;
	uhword_type uhwResult = RESULT_ERROR;
	uword_type uwAltitudeMin = 0;
	uword_type uwAltitudeMax = 0;
	uword_type uwCnt = 0;
	bool_type blTblHit = FALSE;

	//ズームレベル⇒高度変換テーブル
	struct ConvZLvlToAlt stConvZLvlToAltTbl[ALT_CONV_TBL_MAX] ={
		{ZOOM_LEVEL_18, m_lZLvl18_Alt_Min, m_lZLvl18_Alt_Max},
		{ZOOM_LEVEL_19, m_lZLvl19_Alt_Min, m_lZLvl19_Alt_Max},
		{ZOOM_LEVEL_20, m_lZLvl20_Alt_Min, m_lZLvl20_Alt_Max},
		{ZOOM_LEVEL_21, m_lZLvl21_Alt_Min, m_lZLvl21_Alt_Max}
	};

	for(uwCnt = 0; uwCnt < ALT_CONV_TBL_MAX; uwCnt ++){
		if(m_uwZoomlevel == stConvZLvlToAltTbl[uwCnt].uwInZoomLevel){
			uwAltitudeMin = stConvZLvlToAltTbl[uwCnt].uwOutAltMin;
			uwAltitudeMax = stConvZLvlToAltTbl[uwCnt].uwOutAltMax;
			blTblHit = TRUE;
			break;
		}
	}

	//変換対象外の場合初期値で動作
	if(blTblHit == FALSE){
		uwAltitudeMin = m_lZLvl18_Alt_Min;
		uwAltitudeMax = m_lZLvl18_Alt_Max;
	}

	//サービスポート接続確認
	if(blAddDBReqCoordDataGetConsChk()){
		//座標指定画像付加データ取得要求
		uhwServResult = m_CoordinateDataGetCons->usSetCoordinateDataGet(m_uwRobotCategory, m_uwGroupId, m_dStartLatitude, m_dStartLongitude, m_dEndLatitude, m_dEndLongitude, uwAltitudeMin, uwAltitudeMax);
		if(uhwServResult == RESULT_ACK){
			std::cout << "[debug]usSetCoordinateDataGet()" << 
				",Param1 = " << m_uwRobotCategory << ",Param2 = " << m_uwGroupId<< ",Param3 = " << m_dStartLatitude << 
				",Param4 = " << m_dStartLongitude << ",Param5 = " << m_dEndLatitude << ",Param6 = " << m_dEndLongitude <<
				",Param7 = " << uwAltitudeMin << ",Param8 = " << uwAltitudeMax << 
			std::endl;
			uhwResult = RESULT_SUCCESS;
		}
		else{
			//エラーメッセージ
			std::cout << "[error]usSetCoordinateDataGet()" << 
				",Param1 = " << m_uwRobotCategory << ",Param2 = " << m_uwGroupId<< ",Param3 = " << m_dStartLatitude << 
				",Param4 = " << m_dStartLongitude << ",Param5 = " << m_dEndLatitude << ",Param6 = " << m_dEndLongitude <<
				",Param7 = " << uwAltitudeMin << ",Param8 = " << uwAltitudeMax << 
			std::endl;
		}
	}

	return uhwResult;
}

/*****************************************************************************************
*	Function Name		：uhwProcTileSplit
*	Description			：画像を分割する
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwProcTileSplit(void)
{
	uhword_type uhwResult = RESULT_ERROR;
	uword_type uwAddDataNum = 0;
	std::vector<std::string>strAddData;
	uhword_type uhwLoop = 0;
	CalcCoord_st stCalcCoord;
	uhword_type uhwServResult = RESULT_NACK;

	memset(&stCalcCoord, 0, sizeof(stCalcCoord));

	//画像付加データ数を取得
	if((m_uwAddDBNotice == ADD_DB_NOTICE_COORD_GET_SUCCESS) && (m_uwAddDBParam > GROUP_ID_NON) &&(m_uwAddDBParam <= PLAN_LIST_NUM_MAX)){
		uwAddDataNum = m_uwAddDBParam;
	}
	else{
		std::cout << "[warning]The image with the specified coordinates is not in the DB!!" << std::endl;
		TileGenerateNoticeSet(TILEGENE_NOTICE_GENERATE_ERROR, m_uwGenerateMode);
		TileGeneModeSet(MODE_IDLE);
		return RESULT_SUCCESS;
	}

	//Vector領域確保
	strAddData.reserve(uwAddDataNum);
	for(uhwLoop = 0; uhwLoop < uwAddDataNum; uhwLoop++){
		strAddData.push_back("");
	}

	//画像付加データ取得
	if(m_inCoordAdditinalDataIn.isNew()){
		m_inCoordAdditinalDataIn.read();
		for(uhwLoop = 0; uhwLoop < uwAddDataNum; uhwLoop++){
			strAddData[uhwLoop] = m_inCoordAdditinalData.data[uhwLoop];
			//画像付加データ範囲チェック及びメンバ変数へ格納
			uhwResult = uhwAddDataRamSet(strAddData[uhwLoop], uhwLoop);
			if(uhwResult == RESULT_ERROR){
				return RESULT_ERROR;
			}
		}

		//画像のリサイズ
		uhwResult = uhwImgResize();
		if(uhwResult == RESULT_SUCCESS){
			//分割確認画面表示
			if(m_uwGenerateMode == TILE_MODE_SPLIT_DISP_ON){
				uhwResult = uhwSplitImgDisp();
				TileGeneModeSet(MODE_SPLIT_DISP);
				//分割画面表示はこのタイミングで正常終了を返す
				TileGenerateNoticeSet(TILEGENE_NOTICE_GENERATE_SUCCESS, m_uwGenerateMode);
			}
			else{
				//入力画像の起点終点座標を算出
				uhwResult = uhwInputImageCoordCalc(stCalcCoord);
				if(uhwResult == RESULT_SUCCESS){
					//画像の分割＋jsonファイル作成
					uhwResult = uhwSplitImgCreate(stCalcCoord);
					if(uhwResult == RESULT_SUCCESS){
						//地図データDB登録要求
						uhwResult = uhwMapDBReqRegist();
						TileGeneModeSet(MODE_MAPDB_REGIST);
					}
				}
			}
		}
	}

	return uhwResult;
}

/*****************************************************************************************
*	Function Name		：uhwProcTileGenerateComp
*	Description			：タイル生成完了
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwProcTileGenerateComp(void)
{
	TileGenerateNoticeSet(TILEGENE_NOTICE_GENERATE_SUCCESS, m_uwGenerateMode);
	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwProcTileGenerateError
*	Description			：タイル生成エラー
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwProcTileGenerateError(void)
{
	TileGenerateNoticeSet(TILEGENE_NOTICE_GENERATE_ERROR, m_uwGenerateMode);
	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwProcSplitDispRenew
*	Description			：分割確認画面更新
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwProcSplitDispRenew(void)
{
	uhword_type uhwResult = RESULT_ERROR;

	uhwResult = uhwSplitImgDisp();
	return uhwResult;
}

/*****************************************************************************************
*	Function Name		：uhwProcSplitDispOff
*	Description			：分割確認画面消去
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了
******************************************************************************************/
uhword_type TileGenerator::uhwProcSplitDispOff(void)
{
	cv::destroyAllWindows();
	//分割画面表示はこのタイミングで正常終了を返す
	TileGenerateNoticeSet(TILEGENE_NOTICE_GENERATE_SUCCESS, m_uwGenerateMode);
	return RESULT_SUCCESS;
}

/******************************************************************************************
*	Function Name		：blAddDBReqCoordDataGetConsChk
*	Description			：画像付加データDB制御RTCの座標指定付加データ取得要求用の
*						：	サービスポート接続確認
*	Argument			：なし
*	Return				：TRUE：接続正常/FALSE：接続異常
******************************************************************************************/
bool_type TileGenerator::blAddDBReqCoordDataGetConsChk(void)
{
	bool_type ret = TRUE;

	try{
		if(CORBA::is_nil(m_CoordinateDataGetCons._ptr())){
			std::cout << "[nil]m_CoordinateDataGetCons is not assigned" << std::endl;
			return FALSE;
		}
		else{
			if(m_CoordinateDataGetCons->_non_existent()){
				std::cout << "[inactive]m_CoordinateDataGetCons is inactive" << std::endl;
				return FALSE;
			}
			else{
				//サービスポート接続正常
			}
		}
	}
	catch(...){
		std::cout << "[error]Unknown exception" << std::endl;
		return FALSE;
	}

	return ret;
}

/******************************************************************************************
*	Function Name		：blAddDBNoticeConsChk
*	Description			：画像付加データDB制御RTCの完了通知用のサービスポート接続確認
*	Argument			：なし
*	Return				：TRUE：接続正常/FALSE：接続異常
******************************************************************************************/
bool_type TileGenerator::blAddDBNoticeConsChk(void)
{
	bool_type ret = TRUE;

	try{
		if(CORBA::is_nil(m_AddDBNoticeCons._ptr())){
			std::cout << "[nil]m_AddDBNoticeCons is not assigned" << std::endl;
			return FALSE;
		}
		else{
			if(m_AddDBNoticeCons->_non_existent()){
				std::cout << "[inactive]m_AddDBNoticeCons is inactive" << std::endl;
				return FALSE;
			}
			else{
				//サービスポート接続正常
			}
		}
	}
	catch(...){
		std::cout << "[error]Unknown exception" << std::endl;
		return FALSE;
	}

	return ret;
}

/******************************************************************************************
*	Function Name		：blMapDBReqRegistConsChk
*	Description			：地図データDB制御RTCの登録用のサービスポート接続確認
*	Argument			：なし
*	Return				：TRUE：接続正常/FALSE：接続異常
******************************************************************************************/
bool_type TileGenerator::blMapDBReqRegistConsChk(void)
{
	bool_type ret = TRUE;

	try{
		if(CORBA::is_nil(m_MapDBReqRegistAddDataCons._ptr())){
			std::cout << "[nil]m_MapDBReqRegistAddDataCons is not assigned" << std::endl;
			return FALSE;
		}
		else{
			if(m_MapDBReqRegistAddDataCons->_non_existent()){
				std::cout << "[inactive]m_MapDBReqRegistAddDataCons is inactive" << std::endl;
				return FALSE;
			}
			else{
				//サービスポート接続正常
			}
		}
	}
	catch(...){
		std::cout << "[error]Unknown exception" << std::endl;
		return FALSE;
	}

	return ret;
}

/******************************************************************************************
*	Function Name		：blMapDBNoticeConsChk
*	Description			：地図データDB制御RTCの完了通知用のサービスポート接続確認
*	Argument			：なし
*	Return				：TRUE：接続正常/FALSE：接続異常
******************************************************************************************/
bool_type TileGenerator::blMapDBNoticeConsChk(void)
{
	bool_type ret = TRUE;

	try{
		if(CORBA::is_nil(m_MapDataDBNoticeCons._ptr())){
			std::cout << "[nil]m_MapDataDBNoticeCons is not assigned" << std::endl;
			return FALSE;
		}
		else{
			if(m_MapDataDBNoticeCons->_non_existent()){
				std::cout << "[inactive]m_MapDataDBNoticeCons is inactive" << std::endl;
				return FALSE;
			}
			else{
				//サービスポート接続正常
			}
		}
	}
	catch(...){
		std::cout << "[error]Unknown exception" << std::endl;
		return FALSE;
	}

	return ret;
}

/*****************************************************************************************
*	Function Name		：uhwAddDataRamSet
*	Description			：画像付加データの範囲チェックを行う
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwAddDataRamSet(std::string strAddData, uhword_type uhwCnt)
{
	uhword_type uhwResult = RESULT_ERROR;
	Json::Reader reader;
	Json::Value	Val;
	ImgAddDBAddData_st stImgAddData;
	std::string strAddDataRep = "";
	std::string strConvSjis = "";

	if((strAddData != "") && (uhwCnt < PLAN_LIST_NUM_MAX)){
		strAddDataRep = strImgJsonDataReplace(strAddData);

		reader.parse(strAddDataRep, Val);
		
		std::string strFolder = Val[IMGADDDB_FOLDER_PATH].asString();

		//jsonパーサーを使用して付加データを読み込む
		//reader.parse(strAddData, Val);
		reader.parse(strConvSjis, Val);

		//文字列をutf-8⇒sjisに変換
		strConvSjis = strUTF8toSjis(Val[IMGADDDB_FILE_NAME].asString());
		if(strConvSjis == ""){
			return RESULT_ERROR;
		}
		stImgAddData.strFileName = strConvSjis;

		strConvSjis = strUTF8toSjis(Val[IMGADDDB_FOLDER_PATH].asString());
		if(strConvSjis == ""){
			return RESULT_ERROR;
		}
		stImgAddData.strFolderPath = strConvSjis;

		strConvSjis = strUTF8toSjis(Val[IMGADDDB_ROBOT_IDENTIFIER].asString());
		if(strConvSjis == ""){
			return RESULT_ERROR;
		}
		stImgAddData.strRobotIdentifier = strConvSjis;

		strConvSjis = strUTF8toSjis(Val[IMGADDDB_SHOOTING_TIME].asString());
		if(strConvSjis == ""){
			return RESULT_ERROR;
		}
		stImgAddData.strShootingTime = strConvSjis;

		strConvSjis = strUTF8toSjis(Val[IMGADDDB_SHOOTING_PLAN_NAME].asString());
		if(strConvSjis == ""){
			return RESULT_ERROR;
		}
		stImgAddData.strShootingPlanName = strConvSjis;

		stImgAddData.uhwImgWidth = (uhword_type)Val[IMGADDDB_IMAGE_WIDTH].asUInt();
		stImgAddData.uhwImgHight = (uhword_type)Val[IMGADDDB_IMAGE_HEIGHT].asUInt();
		stImgAddData.uhwCameraNo = (uhword_type)Val[IMGADDDB_CAMERA_NO].asUInt();
		stImgAddData.dLatitude = Val[IMGADDDB_LATITUDE].asDouble();
		stImgAddData.dLongitude = Val[IMGADDDB_LONGITUDE].asDouble();
		stImgAddData.dAltitude = Val[IMGADDDB_ALTITUDE].asDouble();
		stImgAddData.fdirection = Val[IMGADDDB_DIRECTION].asFloat();
		stImgAddData.fPitchAngle = Val[IMGADDDB_PITCH].asFloat();
		stImgAddData.fRollAngle = Val[IMGADDDB_ROLL].asFloat();

		//範囲チェック
		uhwResult = uhwImgAddDataRangeCheck(stImgAddData);
		if(uhwResult == RESULT_SUCCESS){
			m_stImgAddData[uhwCnt].strFileName = stImgAddData.strFileName;
			m_stImgAddData[uhwCnt].strFolderPath = stImgAddData.strFolderPath;
			m_stImgAddData[uhwCnt].uhwImgWidth = stImgAddData.uhwImgWidth;
			m_stImgAddData[uhwCnt].uhwImgHight = stImgAddData.uhwImgHight;
			m_stImgAddData[uhwCnt].strRobotIdentifier = stImgAddData.strRobotIdentifier;
			m_stImgAddData[uhwCnt].uhwCameraNo = stImgAddData.uhwCameraNo;
			m_stImgAddData[uhwCnt].dLatitude = stImgAddData.dLatitude;
			m_stImgAddData[uhwCnt].dLongitude = stImgAddData.dLongitude;
			m_stImgAddData[uhwCnt].dAltitude = stImgAddData.dAltitude;
			m_stImgAddData[uhwCnt].fdirection = stImgAddData.fdirection;
			m_stImgAddData[uhwCnt].fPitchAngle = stImgAddData.fPitchAngle;
			m_stImgAddData[uhwCnt].fRollAngle = stImgAddData.fRollAngle;
			m_stImgAddData[uhwCnt].strShootingTime = stImgAddData.strShootingTime;
			m_stImgAddData[uhwCnt].strShootingPlanName = stImgAddData.strShootingPlanName;
		}
	}

	return uhwResult;
}

/*****************************************************************************************
*	Function Name		：uhwImgAddDataRangeCheck
*	Description			：画像付加データの範囲チェックを行う
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwImgAddDataRangeCheck(ImgAddDBAddData_st stImgAddData)
{
	uhword_type uhwLength = 0;

	//ファイル名の文字数チェック
	uhwLength = stImgAddData.strFileName.length();
	if((uhwLength == 0) || (uhwLength > CSV_FILE_NAME_LENGH)){
		std::cout << "[error]Filename is Invalit = " << uhwLength << std::endl;
		return RESULT_ERROR;
	}

	//フォルダパスの文字数チェック
	uhwLength = stImgAddData.strFolderPath.length();
	if((uhwLength == 0) || (uhwLength > CSV_FOLDER_PATH_LENGTH)){
		std::cout << "[error]FolderPath is Invalit = " << uhwLength << std::endl;
		return RESULT_ERROR;
	}

	//画像幅の範囲チェック
	if((stImgAddData.uhwImgWidth == 0) ||
		(stImgAddData.uhwImgWidth > CSV_IMAGE_WIDTH_MAX) ||
		(stImgAddData.uhwImgHight == 0) ||
		(stImgAddData.uhwImgHight > CSV_IMAGE_HEIGHT_MAX)){
		std::cout << "[error]Image Size is Invalit = " << stImgAddData.uhwImgWidth << ", " << stImgAddData.uhwImgHight << std::endl;
		return RESULT_ERROR;
	}

	//ロボット識別子の文字数チェック
	uhwLength = stImgAddData.strRobotIdentifier.length();
	if((uhwLength == 0) || (uhwLength > CSV_ROBOT_IDENTIFIER_LENGH)){
		std::cout << "[error]Robot identifier is Invalit = " << uhwLength << std::endl;
		return RESULT_ERROR;
	}

	//カメラNoの範囲チェック
	if((stImgAddData.uhwCameraNo == 0) || (stImgAddData.uhwCameraNo > CSV_CAMERA_NO_MAX)){
		std::cout << "[error]Camera no is Invalit = " << stImgAddData.uhwCameraNo << std::endl;
		return RESULT_ERROR;
	}

	//緯度の範囲チェック
	if(stImgAddData.dLatitude == 0){
		std::cout << "[error]Latitude is Invalit = " << stImgAddData.dLatitude << std::endl;
		return RESULT_ERROR;
	}

	//経度の範囲チェック
	if(stImgAddData.dLongitude == 0){
		std::cout << "[error]Longitude is Invalit = " << stImgAddData.dLongitude << std::endl;
		return RESULT_ERROR;
	}

	//方角の範囲チェック
	if((stImgAddData.fdirection < CSV_ROBOT_DIRECTION_MIN) || (stImgAddData.fdirection > CSV_ROBOT_DIRECTION_MAX)){
		std::cout << "[error]Direction is Invalit = " << stImgAddData.fdirection << std::endl;
		return RESULT_ERROR;
	}

	//ピッチ角の範囲チェック
	if((stImgAddData.fPitchAngle < CSV_ROBOT_PITCH_MIN) || (stImgAddData.fPitchAngle > CSV_ROBOT_PITCH_MAX)){
		std::cout << "[error]Pitch angle is Invalit = " << stImgAddData.fPitchAngle << std::endl;
		return RESULT_ERROR;
	}

	//ロール角の範囲チェック
	if((stImgAddData.fRollAngle < CSV_ROBOT_ROLL_MIN) || (stImgAddData.fRollAngle > CSV_ROBOT_ROLL_MAX)){
		std::cout << "[error]Roll angle is Invalit = " << stImgAddData.fRollAngle << std::endl;
		return RESULT_ERROR;
	}

	//撮影計画名の文字数チェック
	uhwLength = stImgAddData.strShootingPlanName.length();
	if((uhwLength == 0) || (uhwLength > CSV_SHOOTING_PLANNAME_LENGTH)){
		std::cout << "[error]Shooting plan name is Invalit = " << uhwLength << std::endl;
		return RESULT_ERROR;
	}

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：strUTF8toSjis
*	Description			：UTF-8→Sjis変換
*	Argument			：変換元の文字列
*	Return				：正常終了：変換後の文字列、異常終了：空文字列
******************************************************************************************/
std::string TileGenerator::strUTF8toSjis(std::string strUTF8)
{
	std::string strSJis = "";
	uhword_type uhwUnicodeLen = 0;
	uhword_type uhwJisLen = 0;
	uhword_type uhwConvResult = 0;
	uword_type uwLogLine = 0;
	wchar_t* bufUnicode = NULL;
	char* bufShiftJis = NULL;

	if(strUTF8 == ""){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	//Unicodeへ変換後の文字列長取得
	uhwUnicodeLen = MultiByteToWideChar(CP_UTF8,0,strUTF8.c_str(),strUTF8.size() + 1,NULL,0);
	if(uhwUnicodeLen == 0){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	//Unicode文字列のバッファを確保
	bufUnicode = new wchar_t[uhwUnicodeLen];
	if(bufUnicode == NULL){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	//UTF8からUnicodeへ変換
	uhwConvResult = MultiByteToWideChar(CP_UTF8,0,strUTF8.c_str(),strUTF8.size() + 1,bufUnicode,uhwUnicodeLen);
	if(uhwConvResult == 0){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	//ShiftJISへ変換後の文字列長取得
	uhwJisLen = WideCharToMultiByte(CP_THREAD_ACP,0,bufUnicode,-1,NULL,0,NULL,NULL);
	if(uhwJisLen == 0){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	//ShiftJIS文字列のバッファを確保
	bufShiftJis = new char[uhwJisLen];
	if(bufShiftJis == NULL){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	//UnicodeからShiftJISへ変換
	uhwConvResult = WideCharToMultiByte(CP_THREAD_ACP,0,bufUnicode,uhwUnicodeLen,bufShiftJis,uhwJisLen,NULL,NULL);
	if(uhwConvResult == 0){
		uwLogLine = __LINE__;
		goto SJIS_CONV_ERROR;
	}

	strSJis = bufShiftJis;

	delete[] bufUnicode;
	delete[] bufShiftJis;

	return strSJis;

SJIS_CONV_ERROR:
	std::cout << "[error]MapImageSelect::strUTF8toSjis = " << GetLastError() << ",line = " << uwLogLine << std::endl;

	if(bufUnicode != NULL){
		delete[] bufUnicode;
	}
	if(bufShiftJis != NULL){
		delete[] bufShiftJis;
	}

	strSJis = "";
	return strSJis;
}

/*****************************************************************************************
*	Function Name		：strSjistoUTF8
*	Description			：Sjis→UTF-8変換
*	Argument			：変換元の文字列
*	Return				：正常終了：変換後の文字列、異常終了：空文字列
******************************************************************************************/
std::string TileGenerator::strSjistoUTF8(std::string srcSjis)
{
	std::string strUTF8 = "";
	uhword_type uhwUnicodeLen = 0;
	uhword_type uhwUTF8Len = 0;
	uword_type uwLogLine = 0;
	uhword_type uhwConvResult = 0;
	wchar_t* bufUnicode = NULL;
	char* bufUTF8 = NULL;

	if(srcSjis == ""){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	//Unicodeへ変換後の文字列長取得
	uhwUnicodeLen = MultiByteToWideChar(CP_THREAD_ACP, 0, srcSjis.c_str(), srcSjis.size() + 1, NULL, 0);
	if(uhwUnicodeLen == 0){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	//必要な分だけUnicode文字列のバッファを確保
	bufUnicode = new wchar_t[uhwUnicodeLen];
	if(bufUnicode == NULL){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	//ShiftJISからUnicodeへ変換
	uhwConvResult = MultiByteToWideChar(CP_THREAD_ACP, 0, srcSjis.c_str(), srcSjis.size() + 1, bufUnicode, uhwUnicodeLen);
	if(uhwConvResult == 0){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	//UTF8へ変換後の文字列長を得る
	uhwUTF8Len = WideCharToMultiByte(CP_UTF8, 0, bufUnicode, uhwUnicodeLen, NULL, 0, NULL, NULL);
	if(uhwUTF8Len == 0){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	//必要な分だけUTF8文字列のバッファを確保
	bufUTF8 = new char[uhwUTF8Len];
	if(bufUTF8 == NULL){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	//UnicodeからUTF8へ変換
	uhwConvResult = WideCharToMultiByte(CP_UTF8, 0, bufUnicode, uhwUnicodeLen, bufUTF8, uhwUTF8Len, NULL, NULL);
	if(uhwConvResult == 0){
		uwLogLine = __LINE__;
		goto UTF8_CONV_ERROR;
	}

	strUTF8 = bufUTF8;

	delete[] bufUnicode;
	delete[] bufUTF8;

	return strUTF8;

UTF8_CONV_ERROR:
	std::cout << "[error]TileGenerator::strSjistoUTF8 = " << GetLastError() << ",line = " << uwLogLine << ",Input = " << srcSjis << std::endl;

	if(bufUnicode != NULL){
		delete[] bufUnicode;
	}
	if(bufUTF8 != NULL){
		delete[] bufUTF8;
	}

	strUTF8 = "";
	return strUTF8;
}

/*****************************************************************************************
*	Function Name		：uhwImgResize
*	Description			：コンフィギュレーション設定値に従い入力画像をリサイズする
*	Argument			：なし
*	Return				：正常終了：変換後の文字列、異常終了：空文字列
******************************************************************************************/
uhword_type TileGenerator::uhwImgResize(void)
{
	std::string strFilePath = "";
	std::ostringstream oss;
	bool_type blRet = FALSE;
	cv::Mat matOutImg;
	cv::Mat matInImg;

	//初期化
	oss.str("");
	oss.clear(std::stringstream::goodbit);

	if((m_fImage_Resize <= 0) || (m_fImage_Resize > 2)){
		std::cout << "[error]Image resize error->Config param error!! = " << m_fImage_Resize << std::endl;
		return RESULT_ERROR;
	}

	oss << ROBOT_BASE_FOLDER_PATH << PICTURE_FOLDER << m_stImgAddData[0].strFolderPath << "/" << m_stImgAddData[0].strFileName;
	strFilePath = oss.str();
	if(strFilePath == ""){
		std::cout << "[error]Image resize error->Folder path is undefined!! = " << strFilePath << std::endl;
		return RESULT_ERROR;
	}

	matInImg = cv::imread(strFilePath, cv::IMREAD_COLOR);
	if(matInImg.empty()){
		std::cout << "[error]Image resize error->Input image is empty!!" << std::endl;
		return RESULT_ERROR;
	}

	cv::resize(matInImg, matOutImg, cv::Size(), m_fImage_Resize, m_fImage_Resize);
	if(matOutImg.empty()){
		std::cout << "[error]Image resize error->Output image is empty!!" << std::endl;
		return RESULT_ERROR;
	}

	blRet = cv::imwrite("resize.jpg", matOutImg);
	if(blRet == FALSE){
		std::cout << "[error]Image resize error->resize.jpg generation failed!!" << std::endl;
		return RESULT_ERROR;
	}

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwSplitImgDisp
*	Description			：入力画像にグリッド線表示を行う
*	Argument			：なし
*	Return				：正常終了：変換後の文字列、異常終了：空文字列
******************************************************************************************/
uhword_type TileGenerator::uhwSplitImgDisp(void)
{
	cv::Mat matInImg;
	cv::Mat matOutImg;
	sword_type swRed = 0;
	sword_type swGreen = 0;
	sword_type swBlue = 0;
	sword_type swX1 = 0;
	sword_type swY1 = 0;
	sword_type swX2 = 0;
	sword_type swY2 = 0;
	sword_type swWidth = TILE_SIZE;
	sword_type swHeight = TILE_SIZE;
	uhword_type uhwLoopX = 0;
	uhword_type uhwLoopY = 0;
	uhword_type uhwLoopXMax = 0;
	uhword_type uhwLoopYMax = 0;

	//グリッド線の色を赤色に設定
	swRed = 255;

	//グリッド線の開始位置をオフセットする
	swX1 = m_lOffset_Width;
	swY1 = m_lOffset_Height;

	matInImg = cv::imread("resize.jpg");
	if(matInImg.empty()){
		std::cout << "[error]Grid line disp error->resize.jpg is empty!!" << std::endl;
		return RESULT_ERROR;
	}
	uhwLoopXMax = (matInImg.cols)/TILE_SIZE;
	uhwLoopYMax = (matInImg.rows)/TILE_SIZE;

	matInImg.copyTo(matOutImg);

	//グレースケール表示
	cv::cvtColor(matOutImg, matOutImg, CV_BGR2GRAY);
	cv::cvtColor(matOutImg, matOutImg, CV_GRAY2BGR);

	for(uhwLoopY = 0; uhwLoopY < uhwLoopYMax; uhwLoopY++){
		for(uhwLoopX = 0;uhwLoopX < uhwLoopXMax;uhwLoopX++){
			swX2 = swX1 + swWidth;
			swY2 = swY1 + swHeight;

			//グリッド線が矩形を超える時はLoopを抜ける
			if((swX2 >= matInImg.cols) || (swY2 >= matInImg.rows)){
				break;
			}

			//画像にグリッド線を追加
			cv::line(matOutImg, cv::Point(swX1, swY1), cv::Point(swX1, swY2), cv::Scalar(swBlue, swGreen, swRed));
			cv::line(matOutImg, cv::Point(swX1, swY2), cv::Point(swX2, swY2), cv::Scalar(swBlue, swGreen, swRed));
			cv::line(matOutImg, cv::Point(swX2, swY2), cv::Point(swX2, swY1), cv::Scalar(swBlue, swGreen, swRed));
			cv::line(matOutImg, cv::Point(swX2, swY1), cv::Point(swX1, swY1), cv::Scalar(swBlue, swGreen, swRed));

			swX1 = swX2;
		}
		swY1 = swY2;
		swX1 = m_lOffset_Width;
	}

	cv::namedWindow("Split Image");
	cv::imshow("Split Image", matOutImg);
	cv::waitKey(10);

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwSplitImgCreate
*	Description			：入力画像にグリッド線表示を行う
*	Argument			：なし
*	Return				：正常終了：変換後の文字列、異常終了：空文字列
******************************************************************************************/
uhword_type TileGenerator::uhwSplitImgCreate(CalcCoord_st stCalcCoord)
{
	cv::Mat matInImg;
	//グリッド線の開始位置をオフセットする
	sword_type swX1 = m_lOffset_Width;
	sword_type swY1 = m_lOffset_Height;
	sword_type swX2 = 0;
	sword_type swY2= 0;
	sword_type swLeft= 0;
	sword_type swTop = 0;
	sword_type swWidth = TILE_SIZE;
	sword_type swHeight = TILE_SIZE;
	uhword_type uhwLoopX = 0;
	uhword_type uhwLoopY = 0;
	uhword_type uhwLoopXMax = 0;
	uhword_type uhwLoopYMax = 0;
	uword_type uwXCoord = m_lTileX_Coord_Start;
	uword_type uwYCoord = m_lTileY_Coord_Start;
	std::string strFolderBasePath = "";
	std::string strFolderPath = "";
	std::ostringstream oss;
	uhword_type uhwResult = RESULT_ERROR;
	DWORD dwFileAttribute = -1;
	bool_type blRet = FALSE;

	m_uhwJsonMakeCnt = 1;

	//入力画像の読み込み
	matInImg = cv::imread("resize.jpg");
	if(matInImg.empty()){
		std::cout << "[error]Tile create error->resize.jpg is empty!!" << std::endl;
		return RESULT_ERROR;
	}

	if((matInImg.cols > CSV_IMAGE_WIDTH_MAX) || (matInImg.rows > CSV_IMAGE_HEIGHT_MAX)){
		std::cout << "[error]Tile create error->resize.jpg is sizeover!!" << std::endl;
		return RESULT_ERROR;
	}

	if((matInImg.cols <= TILE_SIZE) || (matInImg.rows <= TILE_SIZE)){
		std::cout << "[error]Tile create error->can not make tiles because the image size is small = " << matInImg.cols << ", " << matInImg.rows << std::endl;
		return RESULT_ERROR;
	}

	if(((swX1 + swWidth) > matInImg.cols) || ((swY1 + swHeight) > matInImg.rows)){
		std::cout << "[error]Tile create error->can not make tiles because large offset = " << swX1 << ", " << swY1 << std::endl;
		return RESULT_ERROR;
	}


	//ループ回数の設定
	uhwLoopXMax = (matInImg.cols)/TILE_SIZE;
	uhwLoopYMax = (matInImg.rows)/TILE_SIZE;

	//フォルダの生成(X座標フォルダを除く)
	strFolderBasePath = strFolderCreate();
	if(strFolderBasePath == ""){
		std::cout << "[error]Tile create error->tile folder create error!!" << std::endl;
		return RESULT_ERROR;
	}

	//空のjsonファイル及び格納フォルダを作成
	uhwResult = uhwJsonFileCreate();
	if(uhwResult == RESULT_ERROR){
		std::cout << "[error]Tile create error->Json file create error!!" << std::endl;
		return RESULT_ERROR;
	}

	for(uhwLoopY = 0; uhwLoopY < uhwLoopYMax; uhwLoopY++){
		//タイルY座標がオーバーフローしていないかチェックする
		if(uwYCoord == UWORD_TYPE_MAX){
			std::cout << "[error]Tile create error->tile Y coord is over!!" << std::endl;
			return RESULT_ERROR;
		}

		for(uhwLoopX = 0;uhwLoopX < uhwLoopXMax;uhwLoopX++){
			//タイルX座標がオーバーフローしていないかチェックする
			if(uwXCoord == UWORD_TYPE_MAX){
				std::cout << "[error]Tile create error->tile X coord is over!!" << std::endl;
				return RESULT_ERROR;
			}

			swLeft = swX1;
			swTop  = swY1;
			swX2 = swX1 + swWidth;
			swY2 = swY1 + swHeight;

			//分割矩形が入力画像サイズを超える時は処理を抜ける
			if((swX2 >= matInImg.cols) || (swY2 >= matInImg.rows)){
				break;
			}

			//タイルX座標フォルダ生成
			//初期化
			oss.str("");
			oss.clear(std::stringstream::goodbit);
			std::string strXCoord = std::to_string(uwXCoord);
			oss << strFolderBasePath << "/" << strXCoord;
			strFolderPath = oss.str();
			dwFileAttribute = GetFileAttributes(strFolderPath.c_str());
			if(dwFileAttribute == -1){
				//フォルダがなければ生成
				blRet = _mkdir(strFolderPath.c_str());
				if(blRet == TRUE){
					//フォルダ生成失敗
					std::cout << "[error]Tile create error->X coord folder create error!!" << std::endl;
					return RESULT_ERROR;
				}
			}

			cv::Mat matTile = matInImg(cv::Rect(swLeft, swTop, swWidth, swHeight));

			//タイル画像に名称を付けて保存
			//初期化
			oss.str("");
			oss.clear(std::stringstream::goodbit);
			oss << uwYCoord << ".png";
			std::string strFileName = oss.str();

			cv::imwrite((strFolderPath + "/" + strFileName), matTile);

			//Jsondデータを作成する
			uhwResult = uhwJsonDataMake(stCalcCoord, strFolderPath, strFileName, uwXCoord, uwYCoord);
			m_uhwJsonMakeCnt ++;
			if(uhwResult == RESULT_ERROR){
				//フォルダ生成失敗
				std::cout << "[error]Tile create error->json data make error!!" << std::endl;
				return RESULT_ERROR;
			}

			if(uwXCoord < UWORD_TYPE_MAX){
				uwXCoord ++;
			}

			swX1 = swX2;
		}
		swY1 = swY2;
		swX1 = m_lOffset_Width;
		uwXCoord = m_lTileX_Coord_Start;

		if(uwYCoord < UWORD_TYPE_MAX){
			uwYCoord ++;
		}
	}

	//Jsonファイルを書き込む
	uhwResult = uhwJsonFileAdd();
	if(uhwResult == RESULT_ERROR){
		//フォルダ生成失敗
		std::cout << "[error]Tile create error->json file write error!!" << std::endl;
		return RESULT_ERROR;
	}

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：strFolderCreate
*	Description			：タイル格納フォルダ生成
*	Argument			：なし
*	Return				：正常終了：格納フォルダパス、異常終了：空文字
******************************************************************************************/
std::string TileGenerator::strFolderCreate(void)
{
	std::string strFolderPath = "";
	std::string strTime = "";
	DWORD dwFileAttribute = -1;
	uword_type uwSecNo = 1;
	std::ostringstream oss;
	WIN32_FIND_DATA find_dir_data;
	bool_type blRet = FALSE;

	//初期化
	oss.str("");
	oss.clear(std::stringstream::goodbit);

	//フォルダ生成時刻を取得
	strTime = strGetNowDate();
	if(strTime == ""){
		return "";
	}

	//タイルフォルダ作成
	oss << MAP_BASE_FOLDER_PATH << TILE_FOLDER;
	strFolderPath = oss.str();
	dwFileAttribute = GetFileAttributes(strFolderPath.c_str());
	if(dwFileAttribute == -1){
		//フォルダがなければ生成
		blRet = _mkdir(strFolderPath.c_str());
		if(blRet == TRUE){
			//フォルダ生成失敗
			return "";
		}
	}

	//セクションNo付きのフォルダを生成
	oss << strTime << "_" << m_stImgAddData[0].strShootingPlanName << "_";
	strFolderPath = oss.str() + "*";
	//セクションNoを割り当てるため、同名のフォルダの数を数える
	HANDLE hFind = FindFirstFile(strFolderPath.c_str(), &find_dir_data);
	do{
		if(hFind != INVALID_HANDLE_VALUE){
			//フォルダがあればセクションNoをインクリメント
			uwSecNo ++;
		}
		else{
			//フォルダが無ければセクションNoを1に設定
			uwSecNo = 1;
		}
	}while(FindNextFile(hFind, &find_dir_data));
	FindClose(hFind);

	oss << uwSecNo;
	strFolderPath = oss.str();
	dwFileAttribute = GetFileAttributes(strFolderPath.c_str());
	if(dwFileAttribute == -1){
		//フォルダがなければ生成
		blRet = _mkdir(strFolderPath.c_str());
		if(blRet == TRUE){
			//フォルダ生成失敗
			return "";
		}
	}

	//ドローンフォルダ生成
	oss << "/" << AP_FOLDER;
	strFolderPath = oss.str();
	dwFileAttribute = GetFileAttributes(strFolderPath.c_str());
	if(dwFileAttribute == -1){
		//フォルダがなければ生成
		blRet = _mkdir(strFolderPath.c_str());
		if(blRet == TRUE){
			//フォルダ生成失敗
			return "";
		}
	}

	//ズームレベルフォルダ生成
	std::string strZoomLevel = std::to_string(m_uwZoomlevel);
	oss << strZoomLevel;
	strFolderPath = oss.str();
	dwFileAttribute = GetFileAttributes(strFolderPath.c_str());
	if(dwFileAttribute == -1){
		//フォルダがなければ生成
		blRet = _mkdir(strFolderPath.c_str());
		if(blRet == TRUE){
			//フォルダ生成失敗
			return "";
		}
	}

	return strFolderPath;
}

/*****************************************************************************************
*	Function Name		：uhwJsonFileCreate
*	Description			：Jsonファイルファイル作成
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwJsonFileCreate(void)
{
	DWORD dwFileAttribute = -1;
	Json::Value	val;
	bool_type blRet = FALSE;
	HANDLE hFind;
	WIN32_FIND_DATA fd;
	bool_type blFileOut = FALSE;

	//TMPフォルダの有無を確認
	dwFileAttribute = GetFileAttributes(TMP_FOLDER_PATH);
	if(dwFileAttribute == -1){
		//フォルダがなければ生成
		blRet = _mkdir(TMP_FOLDER_PATH);
		if(blRet == TRUE){
			std::cout << "[error]Failed to create tmp folder!!" << std::endl;
			return RESULT_ERROR;
		}
	}

	//Jsonファイルを検索
	hFind = FindFirstFile(MAP_DATA_JSON_FILE_PATH, &fd);
	if(hFind == INVALID_HANDLE_VALUE){
		//ファイルがない
		blFileOut = TRUE;
	}
	else{
		//ファイルが存在する場合削除
		if(DeleteFile(MAP_DATA_JSON_FILE_PATH) == TRUE){
			blFileOut = TRUE;
		}
		else{
			std::cout << "[error]Failed to delete the jsonfile!!" << std::endl;
			FindClose(hFind);
			return RESULT_ERROR;
		}
	}

	FindClose(hFind);

	if(blFileOut){
		// ファイル出力ストリームの初期化
		std::ofstream ofs(MAP_DATA_JSON_FILE_PATH, std::ios::out | std::ios::trunc);
		if(!ofs){
			std::cout << "[error]Failed to open the jsonfile!!" << std::endl;
			return RESULT_ERROR;
		}
	}
	
	m_JsonData.clear();

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwJsonDataMake
*	Description			：Jsonファイルに書き込むデータを作成する
*	Argument			：
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwJsonDataMake(CalcCoord_st stCalcCoord, std::string strFolderPath, std::string strFileName, uword_type uwXCoord, uword_type uwYCoord)
{
//	Json::Value JsonData;
	Json::Value vec(Json::arrayValue);
	vec.append(Json::Value(1));
	vec.append(Json::Value(2));
	std::string strNowDate = "";
	std::string strUTF8Conv = "";

	//タイル画像作成日を取得
	strNowDate = strGetNowDate();
	if(strNowDate == ""){
		return RESULT_ERROR;
	}

	//画像付加データを作成
	//タイル名をjis→utf8へ変換
	strUTF8Conv = strSjistoUTF8(strFileName);
	if(strUTF8Conv == ""){
		return RESULT_ERROR;
	}
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_TILE_NAME] = strUTF8Conv;

	//フォルダパスをjis→utf8へ変換
	//余分なベースパスを削除
	strFolderPath.erase(0, MAP_TILE_FOLDER_PATH_LENGTH);
	strUTF8Conv = strSjistoUTF8(strFolderPath);
	if(strUTF8Conv == ""){
		return RESULT_ERROR;
	}
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_FOLDER_PATH] = strUTF8Conv;

	//作成日をjis→utf8へ変換
	strUTF8Conv = strSjistoUTF8(strNowDate);
	if(strUTF8Conv == ""){
		return RESULT_ERROR;
	}
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_CREATE_DATE] = strUTF8Conv;

	//合成計画をjis→utf8へ変換
	strUTF8Conv = strSjistoUTF8(m_stImgAddData[0].strShootingPlanName);
	if(strUTF8Conv == ""){
		return RESULT_ERROR;
	}
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_PLAN_NAME] = strUTF8Conv;

	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_ZOOM_LEVEL] = (double_type)m_uwZoomlevel;
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_TILECODE_X] = (double_type)uwXCoord;
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_TILECODE_Y] = (double_type)uwYCoord;
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_START_LATITUDE] = stCalcCoord.dStartLatitude;
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_START_LONGITUDE] = stCalcCoord.dStartLongitude;
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_END_LATITUDE] = stCalcCoord.dEndLatitude;
	m_JsonData[std::to_string(m_uhwJsonMakeCnt)][MAPDATA_END_LONGITUDE] = stCalcCoord.dEndLongitude;

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwJsonFileAdd
*	Description			：Jsonファイルに追記する
*	Argument			：地図データ格納ポインタ
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwJsonFileAdd(void)
{
	std::string strJson = "";
	Json::StyledWriter	writer;

	// 追記用に開く
	std::ofstream ofs(MAP_DATA_JSON_FILE_PATH, std::ios::out | std::ios::app);
	if(!ofs){
		std::cout << "[error]Failed to open the jsonfile for additionr!!" << std::endl;
		return RESULT_ERROR;
	}

	//jsonフォーマット変換
	strJson = writer.write(m_JsonData);
	if(strJson == ""){
		std::cout << "[error]Json data is empty!!" << std::endl;
		return RESULT_ERROR;
	}

	//ファイルへの書き出し
	ofs << strJson << std::endl;

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：uhwInputImageCoordCalc
*	Description			：入力画像の起点終点座標を算出する
*	Argument			：地図データ格納ポインタ
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwInputImageCoordCalc(CalcCoord_st &stCalcCoord)
{
	uword_type uwTh = 0;
	uword_type uwDistX = 0;
	uword_type uwDistY = 0;
	double_type dLengthLat = 0;
	double_type dLengthLon = 0;

	//画角からΘを算出
	uwTh = 180 - (m_dView_Angle/2) - 90;

	if(uwTh == 0){
		std::cout << "[error]Θ is invalid!!" << std::endl;
		return RESULT_ERROR;
	}

	uwDistX = m_stImgAddData[0].dAltitude/tan(uwTh*(M_PI/180));
	uwDistY = ((3000 * uwDistX * 2) / 4000) / 2;

	dLengthLat = round(uwDistX/30.827);
	dLengthLon = round(uwDistY/24.725);

	//緯度を10進法変換
	uword_type uwTimeLat = 0;
	uword_type uwMinLat = 0;
	double_type dSecLat = 0;
	CoordHextoDec(m_stImgAddData[0].dLatitude, uwTimeLat, uwMinLat, dSecLat);

	//経度を10進法変換
	uword_type uwTimeLon = 0;
	uword_type uwMinLon = 0;
	double_type dSecLon = 0;
	CoordHextoDec(m_stImgAddData[0].dLongitude, uwTimeLon, uwMinLon, dSecLon);

	//60進法に変換し起点座標設定（秒の位置をづらす）
	stCalcCoord.dStartLatitude = dCoordDectoHex(uwTimeLat, uwMinLat, (dSecLat + dLengthLat));
	stCalcCoord.dStartLongitude = dCoordDectoHex(uwTimeLon, uwMinLon, (dSecLon - dLengthLon));

	//60進法に変換し終点座標設定（秒の位置をづらす）
	stCalcCoord.dEndLatitude = dCoordDectoHex(uwTimeLat, uwMinLat, (dSecLat - dLengthLat));
	stCalcCoord.dEndLongitude = dCoordDectoHex(uwTimeLon, uwMinLon, (dSecLon + dLengthLon));

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：CoordHextoDec
*	Description			：GPS座標60進法→10進法変換
*	Argument			：in：60進の座標データ、out；度、分、秒格納ポインタ
*	Return				：なし
******************************************************************************************/
void TileGenerator::CoordHextoDec(double_type dCoordHex, uword_type &uwTime, uword_type &uwMin, double_type &dSec)
{
	uwTime = (uword_type)dCoordHex;
	double_type dWork = dCoordHex - uwTime;
	uwMin = (uword_type)(dWork * 60);
	dSec = ((dWork * 60) - uwMin) * 60;
}

/*****************************************************************************************
*	Function Name		：dCoordDectoHex
*	Description			：GPS座標10進法→60進法変換
*	Argument			：in：度、分、秒
*	Return				：10進に変換した座標データ
******************************************************************************************/
double_type TileGenerator::dCoordDectoHex(uword_type uwTime, uword_type uwMin, double_type dSec)
{
	double_type dWorkMin = 0;
	double_type dWorkSec = 0;
	double_type dResult = 0;

	dWorkMin = ((double_type)uwMin / 60);
	dWorkSec = ((dSec / 60) / 60);
	dResult = uwTime + dWorkMin + dWorkSec;

	return dResult;
}

/*****************************************************************************************
*	Function Name		：strGetNowDate
*	Description			：現在の日付を文字列で返す
*	Argument			：なし
*	Return				：正常終了：日付の文字列、異常終了：空文字列
******************************************************************************************/
std::string TileGenerator:: strGetNowDate(void)
{
	uhword_type uhwLength = 0;

	struct tm *date;
	time_t now;
	char str[256] ={'\0'};
	std::string strNowDate = "";

	time(&now);
	date = localtime(&now);
	if(strftime(str, 255, "%Y%m%d", date) != 0){
		strNowDate = str;
	}

	uhwLength = strNowDate.length();

	if((uhwLength == 0) || (uhwLength > 8)){
		return ("");
	}

	return strNowDate;
}

/*****************************************************************************************
*	Function Name		：uhwMapDBReqRegist
*	Description			：地図データDBへ登録要求を行う
*	Argument			：なし
*	Return				：RESULT_SUCCESS:正常終了、RESULT_ERROR：異常終了
******************************************************************************************/
uhword_type TileGenerator::uhwMapDBReqRegist(void)
{
	uhword_type uhwResult = RESULT_ERROR;
	uhword_type uhwServResult = RESULT_NACK;

	if(blMapDBReqRegistConsChk()){
		uhwServResult = m_MapDBReqRegistAddDataCons->usSetMapDBReqRegistAddData(MAP_DATA_JSON_FILE_PATH);
	}
	if(uhwServResult == RESULT_ACK){
		std::cout << "[debug]usSetMapDBReqRegistAddData()" << ",Param1 = " << MAP_DATA_JSON_FILE_PATH << std::endl;
		uhwResult = RESULT_SUCCESS;
	}
	else{
		std::cout << "[error]usSetMapDBReqRegistAddData()" << ",Param1 = " << MAP_DATA_JSON_FILE_PATH << std::endl;
	}

	return uhwResult;
}

/*****************************************************************************************
*	Function Name		：TileGeneratorRamInit
*	Description			：メンバ変数初期化
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void TileGenerator::TileGeneratorRamInit(void)
{
	uhword_type uhwCnt = 0;

	m_uhwMatrixTriger = TRG_NON;
	m_uhwMatrixMode = MODE_IDLE;
	m_uhwMatrixActID = ACT_NON;

	m_uwAddDBNotice = NOTICE_INIT;
	m_uwAddDBParam = NOTICE_PARAM_INIT;
	m_uwMapDBNotice = NOTICE_INIT;
	m_uwMapDBParam = NOTICE_PARAM_INIT;

	m_uwRobotCategory = ROBOT_CATEGORY_INIT;
	m_uwGroupId = GROUP_ID_NON;
	m_dStartLatitude = 0;
	m_dStartLongitude = 0;
	m_dEndLatitude = 0;
	m_dEndLongitude = 0;
	m_uwZoomlevel = ZOOM_LEVEL_DEF;

	for(uhwCnt = 0; uhwCnt < PLAN_LIST_NUM_MAX ; uhwCnt ++){
		m_stImgAddData[uhwCnt].strFileName = "";
		m_stImgAddData[uhwCnt].strFolderPath = "";
		m_stImgAddData[uhwCnt].uhwImgWidth = 0;
		m_stImgAddData[uhwCnt].uhwImgHight = 0;
		m_stImgAddData[uhwCnt].strRobotIdentifier = "";
		m_stImgAddData[uhwCnt].uhwCameraNo = 0;
		m_stImgAddData[uhwCnt].dLatitude = 0;
		m_stImgAddData[uhwCnt].dLongitude = 0;
		m_stImgAddData[uhwCnt].dAltitude = 0;
		m_stImgAddData[uhwCnt].fdirection = 0;
		m_stImgAddData[uhwCnt].fPitchAngle = 0;
		m_stImgAddData[uhwCnt].fRollAngle = 0;
		m_stImgAddData[uhwCnt].strShootingTime = "";
		m_stImgAddData[uhwCnt].strShootingPlanName = "";
	}

	m_uhwJsonMakeCnt = 1;

	//サービスポート初期化
	m_TileGenerateReqMakeTileProv.InitTileGenerateReqMakeTile();
	m_TileGenerateNoticeProv.InitTileGenerateNotice();
}

/*****************************************************************************************
*	Function Name		：strImgJsonDataReplace
*	Description			：Jsonデータを置換('→")など
*	Argument			：画像付加データDBから取得したJsonデータ
*	Return				：JsonCppで使用可能なJsonデータ
******************************************************************************************/
std::string TileGenerator::strImgJsonDataReplace(std::string strInputData)
{
	std::string strRepAddData = "";
	std::string strPathRepFrom = "'";
	std::string strPathRepTo = "\"";

	strRepAddData = strInputData;

	//シングルクォーテーションをダブルクォーテーションに置換
	std::string::size_type pos = 0;
	while(pos = strRepAddData.find(strPathRepFrom, pos), pos != std::string::npos) {
		strRepAddData.replace(pos, strPathRepFrom.length(), strPathRepTo);
		pos += strPathRepTo.length();
	}

	strPathRepFrom = " u";
	strPathRepTo = " ";
	pos = 0;
	//エスケープを削除
	while(pos = strRepAddData.find(strPathRepFrom, pos), pos != std::string::npos) {

		strRepAddData.replace(pos, strPathRepFrom.length(), strPathRepTo);
		pos += strPathRepTo.length();
	}

	strRepAddData.erase(1, 1);

	return strRepAddData;
}

/*****************************************************************************************
*	Function Name		：TileGeneModeSet
*	Description			：マトリクスモード更新処理
*	Argument			：更新モード
*	Return				：なし
******************************************************************************************/
void TileGenerator::TileGeneModeSet(uhword_type uhwMode)
{
	m_uhwMatrixMode = uhwMode;
	std::cout << "[debug]Matrix Mode = " << m_uhwMatrixMode << std::endl;
}

/*****************************************************************************************
*	Function Name		：TileGenerateNoticeSet
*	Description			：マトリクスモード更新処理
*	Argument			：更新モード
*	Return				：なし
******************************************************************************************/
void TileGenerator::TileGenerateNoticeSet(uword_type uwNotice, uword_type uwParam)
{
	m_TileGenerateNoticeProv.usSetTileGenerateNotice(uwNotice, uwParam);
	std::cout << "[debug]usSetTileGenerateNotice()" << ",param1 = " << uwNotice << ",param2 = " << uwParam << std::endl;
}

#if 0
bool TileGenerator::decode_unicode_escape_to_utf8(const std::string& src, std::string& dst) {
	std::string result("");
	result.resize(src.size());

	for(size_t i = 0; i < result.size(); ++i) {
		result[i] = '\0';
	}

	size_t result_index = 0;
	bool in_surrogate_pair = false;
	size_t surrogate_buffer = 0;
	for(size_t i = 0; i < src.size(); ++i) {
		if(i + 1 < src.size() && src[i] == '\\' && src[i+1] == 'u') {
			size_t octet = 0;
			{  // calculate octet
				const char* const hex = &src[i + 2];
				for(int j = 0; j < 4; ++j) {
					octet *= 16;
					if('0' <= hex[j] && hex[j] <= '9') {
						octet += static_cast<int>(hex[j] - '0');
					}
					else if('a' <= hex[j] && hex[j] <= 'f') {
						octet += static_cast<int>(hex[j] - 'a' + 10);
					}
					else if('A' <= hex[j] && hex[j] <= 'F') {
						octet += static_cast<int>(hex[j] - 'A' + 10);
					}
					else {
						return false;
					}
				}
			}
			{  // fill up sequence
				char* const sequence = &result[result_index];
				if(in_surrogate_pair) {
					if(0xdc00 <= octet && octet <= 0xdfff) {
						// low surrogate pair
						const size_t joined = surrogate_buffer + (octet & 0x03ff) + 0x10000;
						sequence[0] = (static_cast<char>(joined >> 18)  &  0x3) | 0xf0;
						sequence[1] = (static_cast<char>(joined >> 12)  & 0x3f) | 0x80;
						sequence[2] = (static_cast<char>(joined >> 6)   & 0x3f) | 0x80;
						sequence[3] = (static_cast<char>(joined & 0xff) & 0x3f) | 0x80;
						result_index += 4;
						in_surrogate_pair = false;
					}
					else {
						return false;
					}
				}
				else if(octet < 0x7f) {
					sequence[0] = static_cast<char>(octet)& 0x7f;
					result_index += 1;
				}
				else if(octet < 0x7ff) {
					sequence[0] = (static_cast<char>(octet >> 6) & 0xdf) | 0xc0;
					sequence[1] = (static_cast<char>(octet)& 0x3f) | 0x80;
					result_index += 2;
				}
				else if(0xdbff) {
					// high surrogate pair
					in_surrogate_pair = true;
					surrogate_buffer = (octet & 0x03ff) * 0x400;
				}
				else {
					sequence[0] = (static_cast<char>(octet >> 12) & 0x0f) | 0xe0;
					sequence[1] = (static_cast<char>(octet >> 6)  & 0x3f) | 0x80;
					sequence[2] = (static_cast<char>(octet)& 0x3f) | 0x80;
					result_index += 3;
				}
			}
			i += 5;  // \\uXXXX is 6 bytes, so + 5 here, and + 1 in next loop
		}
		else {  // not unicode
			if(in_surrogate_pair) {
				return false;
			}
			result[result_index] = src[i];
			result_index += 1;
		}
		// next char
	}
	result.resize(result_index);
	dst.swap(result);
	return true;
}
#endif

extern "C"
{
 
  void TileGeneratorInit(RTC::Manager* manager)
  {
    coil::Properties profile(tilegenerator_spec);
    manager->registerFactory(profile,
                             RTC::Create<TileGenerator>,
                             RTC::Delete<TileGenerator>);
  }
  
};


